@@ -1,15 +1,27 @@
 """Contains classes and functions for logging."""
-from __future__ import print_function
-import os
+
+from __future__ import annotations
+
 import json
+from contextlib import suppress
+from pathlib import Path
+from typing import TYPE_CHECKING, Any
+
 import numpy as np
-from .observer import _Tracker
-from .event import Events
 from colorama import Fore, just_fix_windows_console
 
+from bayes_opt.event import Events
+from bayes_opt.observer import _Tracker
+
+if TYPE_CHECKING:
+    from os import PathLike
+
+    from bayes_opt.bayesian_optimization import BayesianOptimization
+
 just_fix_windows_console()
 
-def _get_default_logger(verbose, is_constrained):
+
+def _get_default_logger(verbose: int, is_constrained: bool) -> ScreenLogger:
     """
     Return the default logger.
 
@@ -33,7 +45,7 @@ def _get_default_logger(verbose, is_constrained):
 
 class ScreenLogger(_Tracker):
     """Logger that outputs text, e.g. to log to a terminal.
-    
+
     Parameters
     ----------
     verbose : int
@@ -49,20 +61,20 @@ class ScreenLogger(_Tracker):
     _colour_new_max = Fore.MAGENTA
     _colour_regular_message = Fore.RESET
     _colour_reset = Fore.RESET
-    
-    def __init__(self, verbose=2, is_constrained=False):
+
+    def __init__(self, verbose: int = 2, is_constrained: bool = False) -> None:
         self._verbose = verbose
         self._is_constrained = is_constrained
         self._header_length = None
         super().__init__()
 
     @property
-    def verbose(self):
+    def verbose(self) -> int:
         """Return the verbosity level."""
         return self._verbose
 
     @verbose.setter
-    def verbose(self, v):
+    def verbose(self, v: int) -> None:
         """Set the verbosity level.
 
         Parameters
@@ -73,11 +85,11 @@ class ScreenLogger(_Tracker):
         self._verbose = v
 
     @property
-    def is_constrained(self):
+    def is_constrained(self) -> bool:
         """Return whether the logger is constrained."""
         return self._is_constrained
 
-    def _format_number(self, x):
+    def _format_number(self, x: float) -> str:
         """Format a number.
 
         Parameters
@@ -96,11 +108,11 @@ class ScreenLogger(_Tracker):
 
         if len(s) > self._default_cell_size:
             if "." in s:
-                return s[:self._default_cell_size]
-            return s[:self._default_cell_size - 3] + "..."
+                return s[: self._default_cell_size]
+            return s[: self._default_cell_size - 3] + "..."
         return s
 
-    def _format_bool(self, x):
+    def _format_bool(self, x: bool) -> str:
         """Format a boolean.
 
         Parameters
@@ -112,17 +124,10 @@ class ScreenLogger(_Tracker):
         -------
         A stringified, formatted version of `x`.
         """
-        if 5 > self._default_cell_size:
-            if x:
-                x_ = 'T'
-            else:
-                x_ = 'F'
-        else:
-            x_ = str(x)
-        s = f"{x_:<{self._default_cell_size}}"
-        return s
+        x_ = ("T" if x else "F") if self._default_cell_size < 5 else str(x)
+        return f"{x_:<{self._default_cell_size}}"
 
-    def _format_key(self, key):
+    def _format_key(self, key: str) -> str:
         """Format a key.
 
         Parameters
@@ -136,17 +141,17 @@ class ScreenLogger(_Tracker):
         """
         s = f"{key:^{self._default_cell_size}}"
         if len(s) > self._default_cell_size:
-            return s[:self._default_cell_size - 3] + "..."
+            return s[: self._default_cell_size - 3] + "..."
         return s
 
-    def _step(self, instance, colour=_colour_regular_message):
+    def _step(self, instance: BayesianOptimization, colour: str = _colour_regular_message) -> str:
         """Log a step.
 
         Parameters
         ----------
         instance : bayesian_optimization.BayesianOptimization
             The instance associated with the event.
-            
+
         colour :
             (Default value = _colour_regular_message, equivalent to Fore.RESET)
 
@@ -154,21 +159,20 @@ class ScreenLogger(_Tracker):
         -------
         A stringified, formatted version of the most recent optimization step.
         """
-        res = instance.res[-1]
-        cells = []
+        res: dict[str, Any] = instance.res[-1]
+        keys: list[str] = instance.space.keys
+        # iter, target, allowed [, *params]
+        cells: list[str | None] = [None] * (3 + len(keys))
 
-        cells.append(self._format_number(self._iterations + 1))
-        cells.append(self._format_number(res["target"]))
+        cells[:2] = self._format_number(self._iterations + 1), self._format_number(res["target"])
         if self._is_constrained:
-            cells.append(self._format_bool(res["allowed"]))
+            cells[2] = self._format_bool(res["allowed"])
+        params = res.get("params", {})
+        cells[3:] = [self._format_number(params.get(key, float("nan"))) for key in keys]
 
+        return "| " + " | ".join(colour + x + self._colour_reset for x in cells if x is not None) + " |"
 
-        for key in instance.space.keys:
-            cells.append(self._format_number(res["params"][key]))
-
-        return "| " + " | ".join([colour + cells[i] + self._colour_reset for i in range(len(cells))]) + " |"
-
-    def _header(self, instance):
+    def _header(self, instance: BayesianOptimization) -> str:
         """Print the header of the log.
 
         Parameters
@@ -180,21 +184,20 @@ class ScreenLogger(_Tracker):
         -------
         A stringified, formatted version of the most header.
         """
-        cells = []
-        cells.append(self._format_key("iter"))
-        cells.append(self._format_key("target"))
+        keys: list[str] = instance.space.keys
+        # iter, target, allowed [, *params]
+        cells: list[str | None] = [None] * (3 + len(keys))
 
+        cells[:2] = self._format_key("iter"), self._format_key("target")
         if self._is_constrained:
-            cells.append(self._format_key("allowed"))
+            cells[2] = self._format_key("allowed")
+        cells[3:] = [self._format_key(key) for key in keys]
 
-        for key in instance.space.keys:
-            cells.append(self._format_key(key))
-
-        line = "| " + " | ".join(cells) + " |"
+        line = "| " + " | ".join(x for x in cells if x is not None) + " |"
         self._header_length = len(line)
         return line + "\n" + ("-" * self._header_length)
 
-    def _is_new_max(self, instance):
+    def _is_new_max(self, instance: BayesianOptimization) -> bool:
         """Check if the step to log produced a new maximum.
 
         Parameters
@@ -215,7 +218,7 @@ class ScreenLogger(_Tracker):
             self._previous_max = instance.max["target"]
         return instance.max["target"] > self._previous_max
 
-    def update(self, event, instance):
+    def update(self, event: str, instance: BayesianOptimization) -> None:
         """Handle incoming events.
 
         Parameters
@@ -223,17 +226,16 @@ class ScreenLogger(_Tracker):
         event : str
             One of the values associated with `Events.OPTIMIZATION_START`,
             `Events.OPTIMIZATION_STEP` or `Events.OPTIMIZATION_END`.
-            
+
         instance : bayesian_optimization.BayesianOptimization
             The instance associated with the step.
         """
+        line = ""
         if event == Events.OPTIMIZATION_START:
             line = self._header(instance) + "\n"
         elif event == Events.OPTIMIZATION_STEP:
             is_new_max = self._is_new_max(instance)
-            if self._verbose == 1 and not is_new_max:
-                line = ""
-            else:
+            if self._verbose != 1 or is_new_max:
                 colour = self._colour_new_max if is_new_max else self._colour_regular_message
                 line = self._step(instance, colour=colour) + "\n"
         elif event == Events.OPTIMIZATION_END:
@@ -252,7 +254,7 @@ class JSONLogger(_Tracker):
 
     Parameters
     ----------
-    path : str or bytes or os.PathLike
+    path : str or os.PathLike
         Path to the file to write to.
 
     reset : bool
@@ -260,16 +262,14 @@ class JSONLogger(_Tracker):
 
     """
 
-    def __init__(self, path, reset=True):
-        self._path = path
+    def __init__(self, path: str | PathLike[str], reset: bool = True):
+        self._path = Path(path)
         if reset:
-            try:
-                os.remove(self._path)
-            except OSError:
-                pass
+            with suppress(OSError):
+                self._path.unlink(missing_ok=True)
         super().__init__()
 
-    def update(self, event, instance):
+    def update(self, event: str, instance: BayesianOptimization) -> None:
         """
         Handle incoming events.
 
@@ -278,7 +278,7 @@ class JSONLogger(_Tracker):
         event : str
             One of the values associated with `Events.OPTIMIZATION_START`,
             `Events.OPTIMIZATION_STEP` or `Events.OPTIMIZATION_END`.
-            
+
         instance : bayesian_optimization.BayesianOptimization
             The instance associated with the step.
 
@@ -287,19 +287,15 @@ class JSONLogger(_Tracker):
             data = dict(instance.res[-1])
 
             now, time_elapsed, time_delta = self._time_metrics()
-            data["datetime"] = {
-                "datetime": now,
-                "elapsed": time_elapsed,
-                "delta": time_delta,
-            }
+            data["datetime"] = {"datetime": now, "elapsed": time_elapsed, "delta": time_delta}
 
-            if "allowed" in data: # fix: github.com/fmfn/BayesianOptimization/issues/361
+            if "allowed" in data:  # fix: github.com/fmfn/BayesianOptimization/issues/361
                 data["allowed"] = bool(data["allowed"])
-            
+
             if "constraint" in data and isinstance(data["constraint"], np.ndarray):
                 data["constraint"] = data["constraint"].tolist()
 
-            with open(self._path, "a") as f:
+            with self._path.open("a") as f:
                 f.write(json.dumps(data) + "\n")
 
         self._update_tracker(event, instance)
